import boto3
import requests
from botocore.exceptions import ClientError

from subdomain_takeover_tools.helper.main import bootstrap


def is_valid(name, _):
    return confirm_s3(name)


def confirm_s3(name):
    if name.endswith('.amazonaws.com'):
        return False

    try:
        s3 = boto3.resource('s3')
        s3.meta.client.head_bucket(Bucket=name)
        return False
    except ClientError as e:
        response_code = e.response['Error']['Code']
        if '404' == response_code:
            return _confirm_http_response(name)
        elif '403' == response_code:
            return False


def _confirm_http_response(name):
    try:
        r = requests.head("http://" + name, timeout=30)
        if 'x-amz-error-detail-BucketName' in r.headers:
            return True

        r = requests.get("http://" + name, timeout=30)
        return '<BucketName>' in r.text or 'BucketName: ' in r.text or '"BucketName"' in r.text
    except (requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout):
        return False


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import re

import dns.resolver


def prepare_domain_name(host_in):
    host = re.sub(r'https?://', '', host_in)
    host = re.sub(r'/.*$', '', host)
    host = re.sub(r'\.$', '', host)
    return host


def process_subtake_output(is_valid, line, check, inverse, strict):
    (parts, domain) = line.split('\t\t')
    if ': ]' in parts:
        target = ''
    else:
        (_, target) = parts[1:-2].split(': ')
    check(is_valid, domain, target, inverse, strict)


def resolve_cname(hostname):
    try:
        cnames = dns.resolver.resolve(hostname, 'CNAME')

        if len(cnames) == 0:
            raise ResourceWarning("CNAME not found")
        else:
            return prepare_domain_name(str(cnames[0]))
    except dns.resolver.NXDOMAIN:
        return None



import dns.resolver

from subdomain_takeover_tools.helper.main import bootstrap
from subdomain_takeover_tools.helper.prepare import resolve_cname, prepare_domain_name


def is_valid(hostname, cname):
    if hostname == cname:
        cname = resolve_cname(hostname)

    if cname is None:
        return False

    return confirm_azure_app_service(cname, hostname)


def confirm_azure_app_service(cname, hostname):
    cname = prepare_domain_name(cname)

    if cname.count('.') == 2:
        try:
            dns.resolver.resolve('asuid.' + hostname, 'TXT')
        except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer, dns.exception.Timeout, dns.resolver.NoNameservers):
            return True
    return False


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import re
import sys

import tldextract


def main():
    for line in sys.stdin:
        sys.stdout.write(extract_domain_name(line) + '\n')


def extract_domain_name(subdomain):
    if "(" in subdomain:
        return _handle_pattern(subdomain)
    else:
        return _extract_single_domain_name(subdomain)


def _handle_pattern(subdomain):
    end_result = ''
    result = re.search(r'^(.*\.)\(([a-z.|]+)\)', subdomain)
    if result is None:
        # false positive
        return _extract_single_domain_name(subdomain)
    prefix = result.group(1)
    postfixes = result.group(2).split('|')
    for postfix in postfixes:
        end_result = end_result + _extract_single_domain_name(prefix + postfix) + '\n'

    return end_result.strip()


def _extract_single_domain_name(subdomain):
    subdomain = subdomain.replace('(', '').replace(')', '')
    r = tldextract.extract(subdomain)
    return '.'.join(r[-2:])


if __name__ == "__main__":
    main()



import requests

from subdomain_takeover_tools.helper.main import bootstrap, settings
from subdomain_takeover_tools.helper.prepare import prepare_domain_name

session = requests.Session()
session.auth = (settings['github']['username'], settings['github']['access_token'])
url = "https://api.github.com/repos/%s/%s/pages" % (settings['github']['username'], settings['github']['repo'])


def is_valid(domain, _):
    if domain is None:
        return False

    return confirm_github(domain)


def confirm_github(domain):
    if domain.endswith('.github.com') or domain.endswith('.github.io') or domain.endswith('.githubapp.com'):
        return False

    domain = prepare_domain_name(domain)

    configure_cname_response = session.put(url, json={'cname': domain})
    session.put(url, json={'cname': None})
    return configure_cname_response.status_code == 204


def main():
    session.post(url, json={
        'source': {
            'branch': 'main',
            'path': '/'
        }
    })

    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import sys

from subdomain_takeover_tools.authoritative_resolve import query_authoritative
from subdomain_takeover_tools.extract_domain_names import extract_domain_name

_header = ['service', 'target', 'subdomain', 'domain', 'tld', 'wildcard', 'Still valid', 'Validated',
           'Authorarative resolve', 'Service Label']
_pre_validated = ['azure-app-services', 'azure-trafficmanager', 'azure-ip', 'azure-edge', 'fastly', 'agilecrm',
                  'elasticbeanstalk', 's3 bucket', 'shopify', 'tumblr', 'unclaimed']


def main():
    sys.stdout.write(print_as(_header) + '\n')
    for line in sys.stdin:
        sys.stdout.write(print_as(enrich(line)) + '\n')


def print_as(param):
    return ','.join(param)


def has_wildcard(original, original_resolved):
    wildcard = "iughxdfdfguh." + '.'.join(original.split('.')[1:])
    random_resolved = resolve(wildcard)
    return _compare(original_resolved, random_resolved)


def resolve(original):
    try:
        return query_authoritative(original)
    except Exception:
        return None


def enrich(line):
    (service, target, subdomain) = _split_line(line)
    authoritative_result = resolve(subdomain)
    wildcard = has_wildcard(subdomain, authoritative_result)
    active = still_active(authoritative_result, target)
    service = _narrow_down_service(service, target)
    is_validated = _is_validated(service)

    return [
        service,
        target[:-1],
        subdomain,
        extract_domain_name(subdomain),
        subdomain.split('.')[-1],
        str(wildcard),
        str(active),
        is_validated,
        print_authoritative_result(authoritative_result)
    ]


def print_authoritative_result(authoritative_result):
    if authoritative_result is None:
        return "-"

    return '"' + ";".join(authoritative_result) + '"'


def still_active(authoritative_result, target):
    if authoritative_result is None:
        return None

    return target in authoritative_result


def _is_validated(service):
    if service in _pre_validated:
        return 'TRUE'
    else:
        return ''


def _split_line(line):
    (parts, subdomain) = line.strip().split('\t\t')
    (service, target) = parts[1:-1].split(': ')
    return [service, target, subdomain]


def _narrow_down_service(service, target):
    if target.endswith('.azurewebsites.net.'):
        return 'azure-app-services'
    elif target.endswith('.azureedge.net.'):
        return 'azure-edge'
    elif target.endswith('.cloudapp.azure.com.'):
        return 'azure-ip'
    elif target.endswith('.blob.core.windows.net.'):
        return 'azure-blob-storage'
    elif target.endswith('.azure-api.net.'):
        return 'azure-API'
    elif target.endswith('.trafficmanager.net.'):
        return 'azure-trafficmanager'
    else:
        return service


def _compare(original_resolved, random_resolved):
    if original_resolved is None or random_resolved is None:
        return False

    return _first_result(random_resolved) == _first_result(original_resolved)


def _first_result(resolved):
    if len(resolved) == 0:
        return None

    result_list = []
    for res in resolved:
        result_list.append(str(res))

    result_list.sort()

    return result_list[0]


if __name__ == "__main__":
    main()



from subdomain_takeover_tools.helper.main import bootstrap
from subdomain_takeover_tools.helper.prepare import prepare_domain_name


def is_valid(_, cname):
    if cname is None:
        return False

    return confirm_pantheon(cname)


def confirm_pantheon(cname):
    cname = prepare_domain_name(cname)

    if _check_postfix(cname) and cname.count('.') == 2:
        (prefix, _, _) = cname.split('.')

        return prefix.startswith('dev-') or prefix.startswith('test-') or prefix.startswith(
            'test2-') or prefix.startswith('live-') or prefix.startswith('stage-') or prefix.startswith('pre-live-')
    return False


def _check_postfix(cname):
    return cname.endswith('getpantheon.com') or cname.endswith('gotpantheon.com') or cname.endswith('pantheonsite.io')


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import requests

from subdomain_takeover_tools.helper.main import bootstrap, settings
from subdomain_takeover_tools.helper.prepare import prepare_domain_name

session = requests.Session()
session.headers['Fastly-Key'] = settings['fastly']['api_token']
url = "https://api.fastly.com/service/%s/version/%s/domain" % (
    settings['fastly']['service'], settings['fastly']['version']
)


def is_valid(domain, _):
    if domain is None:
        return False

    return confirm_fastly(domain)


def confirm_fastly(domain):
    setup_response = session.post(url, data={'name': domain})
    session.delete(url + '/' + prepare_domain_name(domain))
    return setup_response.status_code == 200


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



from azure.mgmt.trafficmanager import TrafficManagerManagementClient

from subdomain_takeover_tools.helper.credential_wrapper import CredentialWrapper
from subdomain_takeover_tools.helper.main import bootstrap, settings
from subdomain_takeover_tools.helper.prepare import resolve_cname

TRAFFICMANAGER_NET = '.trafficmanager.net'

credential = CredentialWrapper()
tm_client = TrafficManagerManagementClient(credential, settings['azure']['subscription_id'])


def is_valid(hostname, cname):
    if hostname == cname:
        cname = resolve_cname(hostname)

    if cname is None:
        return False

    return confirm_azure_traffic_manager(cname)


def confirm_azure_traffic_manager(cname):
    if TRAFFICMANAGER_NET in cname:
        dns_prefix = cname.replace(TRAFFICMANAGER_NET, '')
        result = tm_client.profiles.check_traffic_manager_relative_dns_name_availability(
            dns_prefix,
            "microsoft.network/trafficmanagerprofiles"
        )
        return result.name_available
    return False


def main():
    bootstrap(is_valid)


if __name__ == "__main__":
    main()



import os
import sys

from setuptools import setup, find_packages


def get_metadata():
    import re
    with open(os.path.join("subdomain_takeover_tools", "__init__.py")) as f:
        return dict(re.findall("__([a-z]+)__ = ['\"]([^'\"]+)['\"]", f.read()))


def update_version(target_version):
    with open(os.path.join("subdomain_takeover_tools", "__init__.py"), 'w') as f:
        for key, value in metadata.items():
            if key == 'version':
                value = target_version

            f.write("__%s__ = '%s'\n" % (key, value))


metadata = get_metadata()

if sys.argv[-1] == 'publish':
    # os.system('cd docs && make html')
    os.system('rm -rf dist')
    os.system('rm -rf build')
    os.system('python3 -m build')

    sys.stdout.write('Continue with publishing and committing? ')
    input()

    print("Uploading release to pypi:")
    os.system("python3 -m twine upload --repository pypi dist/subdomain_takeover_tools-%s* " % metadata['version'])

    print("Tagging:")
    os.system("git tag -a %s -m 'version %s'" % (metadata['version'], metadata['version']))
    os.system("git push --tags")

    print("Upgrading dev version:")
    from bump import SemVer

    version = SemVer.parse(metadata['version'])
    version.bump(minor=True, reset=True)
    update_version(str(version))
    os.system("git add subdomain_takeover_tools/__init__.py")
    os.system("git commit -m \"New development version\"")
    os.system("git push")

    sys.exit()

try:
    long_description = open("README.md", "r").read()
except Exception:
    long_description = None

setup(
    name='subdomain_takeover_tools',
    version=metadata['version'],
    packages=find_packages(),
    url='https://github.com/martinvw/subdomain-takeover-tools',
    license='MIT',
    author='Martin van Wingerden',
    author_email='info@martinvw.nl',
    description='Some helper subdomain_takeover_tools to validate subdomain takeovers',
    long_description=long_description,
    long_description_content_type='text/markdown',
    entry_points={'console_scripts': ['confirm_elb=subdomain_takeover_tools.confirm_elb:main',
                                      'confirm_s3=subdomain_takeover_tools.confirm_s3:main',
                                      'confirm_shopify=subdomain_takeover_tools.confirm_shopify:main',
                                      'confirm_azure_app_service=subdomain_takeover_tools.confirm_azure_app_service'
                                      ':main',
                                      'confirm_azure_traffic_manager=subdomain_takeover_tools'
                                      '.confirm_azure_traffic_manager:main',
                                      'confirm_pantheon=subdomain_takeover_tools.confirm_pantheon:main',
                                      'confirm_azure_edge_cdn=subdomain_takeover_tools.confirm_azure_edge_cdn:main',
                                      'confirm_unclaimed=subdomain_takeover_tools.confirm_unclaimed:main',
                                      'confirm_agile_crm=subdomain_takeover_tools.confirm_agile_crm:main',
                                      'confirm_fastly=subdomain_takeover_tools.confirm_fastly:main',
                                      'confirm_github=subdomain_takeover_tools.confirm_github:main',
                                      'confirm_tumblr=subdomain_takeover_tools.confirm_tumblr:main',
                                      'confirm_cargo=subdomain_takeover_tools.confirm_cargo:main',
                                      'confirm_surge=subdomain_takeover_tools.confirm_surge:main',
                                      'confirm_takeover=subdomain_takeover_tools.confirm_takeover:main',
                                      'extract_domain_names=subdomain_takeover_tools.extract_domain_names:main',
                                      'authoritative_resolve=subdomain_takeover_tools.authoritative_resolve:main',
                                      'subtake_enrich_and_export=subdomain_takeover_tools.subtake_enrich_and_export'
                                      ':main',
                                      ]},
    include_package_data=True,
    install_requires=[
        'tldextract',
        'dnspython',
        'boto3',
        'azure-identity',
        'azure-mgmt-trafficmanager'
    ],
    extras_require={
        'dev': [
            'bump'
        ]
    },
    keywords=['subdomain-takeover', 'subtake', 'elb', 's3', 'shopify'],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Development Status :: 4 - Beta',
        'Topic :: Terminals',
        'Topic :: Utilities',
    ],
)



import configparser
import os
import sys

import requests
import tldextract

from subdomain_takeover_tools.helper.prepare import prepare_domain_name, process_subtake_output

settings = configparser.ConfigParser()
settings.read(os.path.expanduser('~/.subdomain_takeover_tools.ini'))


def bootstrap(is_valid):
    inverse = '--inverse' in sys.argv
    strict = '--strict' in sys.argv

    data = sys.stdin.read()

    lines = data.strip().split('\n')
    for line in lines:
        if not line.strip():
            continue

        if ']\t\t' in line:
            process_subtake_output(is_valid, line, check, inverse, strict)
        else:
            host = prepare_domain_name(line)
            check(is_valid, host, host, inverse, strict)


def check(is_valid, domain, cname, inverse, strict):
    result = is_valid(domain, cname)

    if result and strict:
        r = tldextract.extract(domain)
        host = '.'.join(r[-2:])

        host_result = True

        if host != domain:
            try:
                valid = is_valid(host, None)
                # print("Checked '%s', result: %s" % (host, valid))
                host_result = host_result and valid
            except requests.exceptions.ConnectionError:
                pass

        if 'www.' + host != domain:
            try:
                valid_www = is_valid('www.' + host, None)
                # print("Checked 'www.%s', result: %s" % (host, valid_www))
                host_result = host_result and valid_www
            except requests.exceptions.ConnectionError:
                pass

        result = result and not host_result

    # xor
    if inverse != result:
        print(domain)
